/* $RCSfile: sdai.h,v $
 * $Revision: 1.11 $ $Date: 2000/10/02 17:34:23 $
 * Auth: Jochen Fritz (jfritz@steptools.com)
 * 
 * 	Copyright (c) 1991-2000 by 
 * 	STEP Tools Inc., Troy, New York
 * 	All Rights Reserved
 * 
 * 	This software is furnished under a license and may be used and
 * 	copied only in accordance with the terms of such license and with
 * 	the inclusion of the above copyright notice.  This software and
 * 	accompanying written materials or any other copies thereof may
 * 	not be provided or otherwise made available to any other person.
 * 	No title to or ownership of the software is hereby transferred.
 * 
 * 		----------------------------------------
 */

/****************************************************************************/
/*     This header file <sdai.h> is related to ISO CD 10303-24              */
/****************************************************************************/

#ifndef _SDAI_
#define _SDAI_

#ifndef NeedFunctionPrototypes	
#if defined(FUNCPROTO) || defined(__STDC__) || defined(__cplusplus)
#define NeedFunctionPrototypes 1
#else
#define NeedFunctionPrototypes 0
#endif /* __STDC__ */
#endif /* NeedFunctionPrototypes */

#ifdef __cplusplus
extern "C" {
#endif

struct Sdai_Id {
    int a;
};
    
struct Sdai_Id_aggr {
    int a;
};
    
struct Sdai_ItrId {
    int a;
};

struct Sdai_ADBId {
    int a;
};
    
    
typedef struct Sdai_Id * SdaiId;
typedef struct Sdai_Id_aggr * SdaiId_aggr;
typedef struct Sdai_ItrId * SdaiItrId;
typedef struct Sdai_ADBId * SdaiADBId;
typedef unsigned SdaiIndexId;
typedef int SdaiErrorId;
/***  Constant declarations
****************************************************/

/* NULL Instance */
#define sdaiNULL 0

/* LOGICAL and BOOLEAN value elements: */

#define sdaiFALSE   0
#define sdaiTRUE    1
#define sdaiUNKNOWN 2

/* BINARY value elements: */

#define sdaiBIT0 '0'
#define sdaiBIT1 '1'

/* EXPRESS built-in constants: */

#define sdaiE  2.7182818284590451353602874713
#define sdaiPI 3.1415926353897932384626433832

/* SDAI C late binding error codes:
 *******************************************/

#define sdaiNO_ERR     0   /* No error */
#define sdaiSS_OPN    10   /* Session open */
#define sdaiSS_NAVL   20   /* SDAI not available*/
#define sdaiSS_NOPN   30   /* Session is not open*/
#define sdaiRP_NEXS   40   /* Repository does not exist*/
#define sdaiRP_NAVL   50   /* Repository not available*/
#define sdaiRP_OPN    60   /* Repository open*/
#define sdaiRP_NOPN   70   /* Repository is not open*/
#define sdaiTR_EAB    80   /* Transaction ended abnormally*/
#define sdaiTR_EXS    90   /* Transaction exists*/
#define sdaiTR_NAVL  100   /* Transaction  currently not available */
#define sdaiTR_RW    110   /* Transaction read-write */
#define sdaiTR_NRW   120   /* Transaction not read-write */
#define sdaiTR_NEXS  130   /* Transaction does not exist */
#define sdaiMO_NDEQ  140   /* SDAI-model not domain equivalent */
#define sdaiMO_NEXS  150   /* SDAI-model does not exist */
#define sdaiMO_NVLD  160   /* SDAI-model invalid */
#define sdaiMO_DUP   170   /* SDAI-model duplicate */
#define sdaiMX_NRW   180   /* SDAI-model access not read-write */
#define sdaiMX_NDEF  190   /* SDAI-model access not defined*/
#define sdaiMX_RW    200   /* SDAI-model access read-write*/
#define sdaiMX_RO    210   /* SDAI-model access read-only*/
#define sdaiSD_NDEF  220   /* Schema definition not defined*/
#define sdaiED_NDEF  230   /* Entity definition not defined*/
#define sdaiED_NDEQ  240   /* Entity  definition not domain equival.  */
#define sdaiED_NVLD  250   /* Entity definition invalid*/
#define sdaiRU_NDEF  260   /* Rule not defined*/
#define sdaiEX_NSUP  270   /* Expression evaluation not supported */
#define sdaiAT_NVLD  280   /* Attribute invalid*/
#define sdaiAT_NDEF  290   /* Attribute not defined*/
#define sdaiSI_DUP   300   /* Schema instance duplicate*/
#define sdaiSI_NEXS  310   /* Schema instance does not exist*/
#define sdaiEI_NEXS  320   /* Entity instance does not exist*/
#define sdaiEI_NAVL  330   /* Entity instance not available*/
#define sdaiEI_NVLD  340   /* Entity instance invalid*/
#define sdaiEI_NEXP  350   /* Entity instance not exported*/
#define sdaiSC_NEXS  360   /* Scope does not exist*/
#define sdaiSC_EXS   370   /* Scope exists*/
#define sdaiAI_NEXS  380   /* Aggregate instance does not exist*/
#define sdaiAI_NVLD  390   /* Aggregate instance invalid*/
#define sdaiAI_NSET  400   /* Aggregate instance is empty*/
#define sdaiVA_NVLD  410   /* Value invalid*/
#define sdaiVA_NEXS  420   /* Value does not exist*/
#define sdaiVA_NSET  430   /* Value not set*/
#define sdaiVT_NVLD  440   /* Value type invalid*/
#define sdaiIR_NEXS  450   /* Iterator does not exist*/
#define sdaiIR_NSET  460   /* Current member is not defined*/
#define sdaiIX_NVLD  470   /* Index invalid*/
#define sdaiER_NSET  480   /* Event recording not set*/
#define sdaiOP_NVLD  490   /* Operator invalid*/
#define sdaiFN_NAVL  500   /* Function not available*/
#define sdaiAB_NEXS  800   /* ADB does not exist*/
#define sdaiSY_ERR  1000   /* Underlying system error*/

/***Type declarations********************************************************/

typedef unsigned char    SdaiBit;

/* C late binding simple data types: */
typedef long        SdaiInteger;
typedef double      SdaiReal;
typedef SdaiReal    SdaiNumber;
typedef int         SdaiBoolean;
typedef int         SdaiLogical;
typedef char*       SdaiString;
typedef SdaiBit*    SdaiBinary;

/* enumeration data type: */
typedef char* 		SdaiEnum;

/* aggregate data types: */
typedef SdaiId_aggr       SdaiAggr;
typedef SdaiAggr          SdaiOrderedAggr;
typedef SdaiAggr          SdaiUnorderedAggr;
typedef SdaiOrderedAggr   SdaiArray;
typedef SdaiOrderedAggr   SdaiList;
typedef SdaiUnorderedAggr SdaiSet;
typedef SdaiUnorderedAggr SdaiBag;

/* attribute type data type: */
typedef enum { sdaiADB, sdaiAGGR, sdaiBINARY, sdaiBOOLEAN,
	       sdaiENUM, sdaiINSTANCE, sdaiINTEGER, sdaiLOGICAL,
	       sdaiNOTYPE, sdaiNUMBER, sdaiREAL, sdaiSTRING }
SdaiPrimitiveType;

/* entity instance identifier type: */
typedef SdaiId      SdaiInstance;

/* SDAI instance identifier types: */
typedef SdaiInstance          SdaiAppInstance;
typedef SdaiInstance          SdaiModel;
typedef SdaiInstance          SdaiRep;
typedef SdaiInstance          SdaiSession;
typedef SdaiInstance          SdaiAttr;
typedef SdaiAttr              SdaiExplicitAttr;
typedef SdaiInstance          SdaiNamedType;
typedef SdaiNamedType         SdaiEntity;
typedef SdaiNamedType         SdaiDefinedType;
typedef SdaiInstance          SdaiWhereRule;
typedef SdaiInstance          SdaiUniRule;
typedef SdaiInstance          SdaiGlobalRule;
typedef SdaiInstance          SdaiSchema;
typedef SdaiInstance          SdaiScope;
typedef SdaiInstance          SdaiSchemaInstance;
typedef SdaiInstance          SdaiTrx;

    /* SDAI iterator identifier type: */
typedef SdaiItrId             SdaiIterator;

/* Non-persistent list data type: */

typedef SdaiList              SdaiNPL;

/* Query source data type: */
typedef enum { sdaiQUERY_AGGR, sdaiQUERY_MODEL, sdaiQUERY_REP,
	       sdaiQUERY_SCHEMAINSTANCE} SdaiQuerySourceType;

/* access mode data type: */
typedef enum { sdaiRO, sdaiRW } SdaiAccessMode;

/* C late binding ADB identifier type: */
typedef SdaiADBId SdaiADB;

/* aggregate index data type: */
typedef SdaiIndexId SdaiAggrIndex;

/* error code data type: */
typedef SdaiErrorId SdaiErrorCode;

/* error handler data type: */
typedef void (*SdaiErrorHandler)(
#if NeedFunctionPrototypes
    SdaiErrorCode
#endif
    );

/* transaction commit mode data type: */
typedef enum { sdaiABORT, sdaiCOMMIT } SdaiCommitMode;



/***************************************************/
/* the ANSI-C style function prototypes of the SDAI operations
 */

SdaiSession sdaiOpenSession (
#if NeedFunctionPrototypes
    void
#endif
    );

int sdaiLogicalCompare (
#if NeedFunctionPrototypes
    SdaiLogical value1, SdaiLogical value2
#endif
    );

SdaiErrorCode sdaiErrorQuery (
#if NeedFunctionPrototypes
    void
#endif
    );

void sdaiSetErrorHandler (
#if NeedFunctionPrototypes
    SdaiErrorHandler function
#endif
    );

SdaiErrorHandler sdaiRestoreErrorHandler (
#if NeedFunctionPrototypes
    void
#endif
    );

#define sdaiIsEqual(inst1,inst2) ((inst1)==(inst2))

void sdaiRecordEvent (
#if NeedFunctionPrototypes
    SdaiSession session,
    SdaiString functionName,
    SdaiErrorCode error,
    SdaiString description
#endif
    );

SdaiBoolean sdaiSetEventRecording (
#if NeedFunctionPrototypes
    SdaiSession session,
    SdaiBoolean setRec
#endif
    );

void sdaiCloseSession (
#if NeedFunctionPrototypes
    SdaiSession session
#endif
    );

SdaiRep sdaiOpenRepository (
#if NeedFunctionPrototypes
    SdaiSession session,
    SdaiRep repository
#endif
    );

SdaiRep sdaiOpenRepositoryBN (
#if NeedFunctionPrototypes
    SdaiSession session,
    SdaiString repositoryName
#endif
    );

/* patch007*/
void _sdaiRescanRepository (
#if NeedFunctionPrototypes
    SdaiRep repository
#endif
    );

SdaiTrx sdaiStartTrx (
#if NeedFunctionPrototypes
    SdaiSession session,
    SdaiAccessMode mode
#endif
    );

void sdaiBreakTrx (
#if NeedFunctionPrototypes
    SdaiTrx transaction,
    SdaiCommitMode mode
#endif
    );

void sdaiEndTrx (
#if NeedFunctionPrototypes
    SdaiTrx transaction,
    SdaiCommitMode mode
#endif
    );

SdaiNPL sdaiCreateNPL (
#if NeedFunctionPrototypes
    void
#endif
    );

void sdaiDeleteNPL (
#if NeedFunctionPrototypes
    SdaiNPL list
#endif
    );

SdaiInteger sdaiQuery (
#if NeedFunctionPrototypes
    SdaiQuerySourceType sourceType,
    SdaiString criteria,
    SdaiInstance instance,		       
    SdaiNPL result,
    ...
#endif
    );

SdaiLogical sdaiIsRecordingOn (
#if NeedFunctionPrototypes
    SdaiSession session
#endif
    );

SdaiADB sdaiCreateADB (
#if NeedFunctionPrototypes
    SdaiPrimitiveType valueType,
    ...
#endif
    );

SdaiADB sdaiCreateEmptyADB (
#if NeedFunctionPrototypes
    void
#endif	
    );

void *sdaiGetADBValue(
#if NeedFunctionPrototypes
    SdaiADB block,
    SdaiPrimitiveType valueType,
    void *value
#endif	
    );

void sdaiPutADBValue (
#if NeedFunctionPrototypes
    SdaiADB block,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

SdaiPrimitiveType sdaiGetADBType (
#if NeedFunctionPrototypes
    SdaiADB block
#endif	
    );

void sdaiUnsetADB (
#if NeedFunctionPrototypes
    SdaiADB block
#endif	
    );

void sdaiDeleteADB (
#if NeedFunctionPrototypes
    SdaiADB block
#endif	
    );

SdaiModel sdaiCreateModel (
#if NeedFunctionPrototypes
    SdaiRep repository,
    SdaiString modelName,
    SdaiSchema schema
#endif	
    );

#define sdaiCreateModelBN(repository, modelName, schemaName) \
sdaiCreateModel(repository, modelName, sdaiGetSchema(schemaName))

void _sdaiAddSchema (
#if NeedFunctionPrototypes
    SdaiModel mod,
    SdaiSchema sch
#endif
    );

void _sdaiGetSchemaList (
#if NeedFunctionPrototypes
    SdaiModel mod,
    SdaiNPL list
#endif
    );

    
SdaiSchemaInstance sdaiCreateSchemaInstance (
#if NeedFunctionPrototypes
	SdaiString schemaInstanceName,
	SdaiSchema schema,
	SdaiRep repository
#endif	
	);

#define sdaiCreateSchemaInstanceBN(schemaInstanceName,schemaName,rep) \
sdaiCreateSchemaInstance(schemaInstanceName,sdaiGetSchema(schemaName),rep)


    void sdaiCloseRepository (
#if NeedFunctionPrototypes
	SdaiRep repository
#endif	
	);

void sdaiDeleteSchemaInstance (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance
#endif	
    );

void sdaiDeleteSchemaInstanceBN (
#if NeedFunctionPrototypes
    SdaiString schemaInstanceName,
    SdaiRep repository
#endif	
    );

void sdaiRenameSchemaInstance (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiString schemaInstanceName
#endif	
    );

void sdaiRenameSchemaInstanceBN (
#if NeedFunctionPrototypes
    SdaiString schemaInstanceOldName,
    SdaiRep repository,
    SdaiString schemaInstanceName
#endif	
    );

void sdaiAddModel (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiModel model
#endif	
    );

void sdaiAddModelBN (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiRep repository, SdaiString modelName
#endif	
    );

void sdaiRemoveModel (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiModel model
#endif	
    );

void sdaiRemoveModelBN (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiRep repository, SdaiString modelName
#endif	
    );


SdaiLogical sdaiValidateGlobalRule (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiGlobalRule rule,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateGlobalRuleBN (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiString ruleName,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateUniqueness (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiUniRule uniRule,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateUniquenessBN (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiString entityName,
    SdaiString uniRuleName,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateReferenceDomain (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance,
    SdaiAppInstance appInstance,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateSchemaInstance (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance
#endif	
    );

SdaiBoolean sdaiIsValidationCurrent (
#if NeedFunctionPrototypes
    SdaiSchemaInstance schemaInstance
#endif	
    );

SdaiSchema sdaiGetSchema (
#if NeedFunctionPrototypes
    SdaiString schemaName
#endif	
    );

SdaiSchemaInstance sdaiGetSchemaInstance (
#if NeedFunctionPrototypes
    SdaiString schemaInstanceName,
    SdaiRep repository
#endif	
    );

void sdaiDeleteModel (
#if NeedFunctionPrototypes
    SdaiModel model
#endif	
    );

/*  patch003*/
void _sdaiSetModelFormat (
#if NeedFunctionPrototypes
    SdaiModel mod,
    SdaiString format
#endif
    );
    
void sdaiRenameModel (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiString modelName
#endif	
    );

SdaiModel sdaiAccessModel (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiAccessMode mode
#endif	
    );

SdaiModel _sdaiFindModelBN (
#if NeedFunctionPrototypes
    SdaiRep repository,
    SdaiString modelName
#endif	
    );
    
SdaiModel sdaiAccessModelBN (
#if NeedFunctionPrototypes
    SdaiRep repository,
    SdaiString modelName,
    SdaiAccessMode mode
#endif	
    );

void sdaiPromoteModel (
#if NeedFunctionPrototypes
    SdaiModel model
#endif	
    );

void sdaiEndModelAccess (
#if NeedFunctionPrototypes
    SdaiModel model
#endif	
    );

SdaiEntity sdaiGetEntity (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiString name
#endif	
    );

SdaiEntity _sdaiGetEntityInModel (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiString name
#endif	
    );
    
SdaiEntity _sdaiGetEntityInSchema (
#if NeedFunctionPrototypes
    SdaiSchema sch,
    SdaiString name
#endif
    );

SdaiAppInstance sdaiCreateInstance (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiEntity entity
#endif	
    );

SdaiAppInstance sdaiCreateInstanceBN (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiString entityName
#endif	
    );

void sdaiUndoChanges (
#if NeedFunctionPrototypes
    SdaiModel model
#endif	
    );

void sdaiSaveChanges (
#if NeedFunctionPrototypes
    SdaiModel model
#endif	
    );

void _sdaiSaveAs (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiString asname
#endif	
    );

    
SdaiAppInstance sdaiCreateComplexInstance (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiNPL entityList
#endif	
    );

SdaiAppInstance sdaiCreateComplexInstanceBN (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiInteger nameNumber,
    SdaiString * nameVector
#endif	
    );

SdaiSet sdaiGetEntityExtent (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiEntity entity
#endif	
    );

SdaiSet sdaiGetEntityExtentBN (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiString name
#endif	
    );

void sdaiAddToScope (
#if NeedFunctionPrototypes
    SdaiAppInstance scopeInstance,
    SdaiAppInstance instance
#endif	
    );

SdaiLogical sdaiIsScopeOwner (
#if NeedFunctionPrototypes
    SdaiAppInstance instance
#endif	
    );

SdaiScope sdaiGetScope (
#if NeedFunctionPrototypes
    SdaiAppInstance instance
#endif	
    );

void sdaiRemoveFromScope (
#if NeedFunctionPrototypes
    SdaiScope scope,
    SdaiAppInstance instance
#endif	
    );

void sdaiAddToExportList (
#if NeedFunctionPrototypes
    SdaiScope scope,
    SdaiAppInstance instance
#endif	
    );

void sdaiRemoveFromExportList (
#if NeedFunctionPrototypes
    SdaiScope scope,
    SdaiAppInstance instance
#endif	
    );

void sdaiScopedDelete (
#if NeedFunctionPrototypes
    SdaiScope scope
#endif	
    );

SdaiScope sdaiScopedCopyInSameModel (
#if NeedFunctionPrototypes
    SdaiScope scope
#endif	
    );

SdaiScope sdaiScopedCopyToOtherModel (
#if NeedFunctionPrototypes
    SdaiScope scope,
    SdaiModel model
#endif	
    );

SdaiLogical sdaiValidateScopeReferenceRestrictions(
#if NeedFunctionPrototypes
    SdaiAppInstance instance
#endif	
    );

SdaiSet   sdaiGetOwnedScopeInstances (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance
#endif	
    );

SdaiAppInstance sdaiGetScopeOwner (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance
#endif	
    );

SdaiSet sdaiGetExportList (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance
#endif	
    );

SdaiEntity sdaiGetComplexEntity (
#if NeedFunctionPrototypes
    SdaiNPL entityList
#endif	
    );

SdaiEntity sdaiGetComplexEntityBN (
#if NeedFunctionPrototypes
    SdaiString schemaName,
    SdaiInteger nameNumber,
    SdaiString *nameVector
#endif	
    );

SdaiEntity _sdaiGetComplexEntityBN (
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiInteger nameNumber,
    SdaiString *nameVector
#endif	
    );

SdaiBoolean sdaiIsSubtypeOf (
#if NeedFunctionPrototypes
    SdaiEntity subtype,
    SdaiEntity supertype
#endif	
    );

SdaiBoolean sdaiIsSubtypeOfBN (
#if NeedFunctionPrototypes
    SdaiString schemaName,
    SdaiString subName,
    SdaiString superName
#endif	
    );

SdaiBoolean sdaiIsSDAISubtypeOf (
#if NeedFunctionPrototypes
    SdaiEntity subtype,
    SdaiEntity supertype
#endif	
    );

SdaiBoolean sdaiIsSDAISubtypeOfBN (
#if NeedFunctionPrototypes
    SdaiString schemaName,
    SdaiString subName,
    SdaiString superName
#endif	
    );

SdaiBoolean sdaiIsDeqWith (
#if NeedFunctionPrototypes
    SdaiEntity entityType1,
    SdaiEntity entityType2
#endif	
    );

SdaiBoolean sdaiIsDeqWithBN (
#if NeedFunctionPrototypes
    SdaiString schemaName1,
    SdaiString entityName1,
    SdaiString schemaName2,
    SdaiString entityName2
#endif	
    );

SdaiAttr sdaiGetAttrDefinition (
#if NeedFunctionPrototypes
    SdaiEntity entity,
    SdaiString attrName
#endif	
    );

SdaiAttr sdaiGetAttrDefinitionBN (
#if NeedFunctionPrototypes
    SdaiString schemaName,
    SdaiString entityName,
    SdaiString attrName
#endif	
    );

void* sdaiGetAttr (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiAttr attribute,
    SdaiPrimitiveType valueType,
    void *value
#endif	
    );

void* sdaiGetAttrBN (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiString attributeName,
    SdaiPrimitiveType valueType,
    void *value
#endif	
    );

SdaiBoolean sdaiTestAttr (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiAttr attribute
#endif	
    );

SdaiBoolean sdaiTestAttrBN(
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiString attrName
#endif	
    );

SdaiModel sdaiGetInstanceModel (
#if NeedFunctionPrototypes
    SdaiInstance instance
#endif	
    );

SdaiEntity sdaiGetInstanceType (
#if NeedFunctionPrototypes
    SdaiInstance instance
#endif	
    );

SdaiBoolean sdaiIsInstanceOf (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiEntity entity
#endif	
    );

SdaiBoolean sdaiIsInstanceOfBN (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiString entityName
#endif	
    );

SdaiBoolean sdaiIsKindOf (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiEntity entity
#endif	
    );

SdaiBoolean sdaiIsKindOfBN (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiString entityName
#endif	
    );

SdaiBoolean sdaiIsSDAIKindOf (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiEntity entity
#endif	
    );

SdaiBoolean sdaiIsSDAIKindOfBN (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiString entityName
#endif	
    );

SdaiNPL sdaiFindInstanceUsers (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiNPL domain,
    SdaiNPL resultList
#endif	
    );

SdaiNPL sdaiFindInstanceUsedIn (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiAttr role,
    SdaiNPL domain,
    SdaiNPL resultList
#endif	
    );

SdaiNPL sdaiFindInstanceUsedInBN (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiString roleName,
    SdaiNPL domain,
    SdaiNPL resultList
#endif	
    );

SdaiInteger sdaiGetAttrBound (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiAttr attribute
#endif	
    );

SdaiInteger sdaiGetAttrBoundBN(
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiString attributeNam
#endif	
    );

SdaiNPL sdaiFindInstanceRolesOf (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiNPL domain,
    SdaiNPL resultList
#endif	
    );

SdaiNPL sdaiFindInstanceTypeOf (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiNPL resultList
#endif	
    );

void sdaiGetAttrs (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiInteger
    numberAttr,
    SdaiAttr attribute,
    SdaiPrimitiveType
    valueType,
    void *value,
    ...
#endif	
    );

void sdaiGetAttrsBN (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiInteger numberAttr,
    SdaiString attributeName,
    SdaiPrimitiveType
    valueType,
    void *value,
    ...
#endif	
    );

SdaiNPL _sdaiGetAllAttrDefinitions (
#if NeedFunctionPrototypes
    SdaiEntity entity,
    SdaiNPL resultList
#endif
    );

/* New */
SdaiNPL _sdaiGetSupertypes (
#if NeedFunctionPrototypes
    SdaiEntity entity,
    SdaiNPL resultList
#endif
    );

/* New */
SdaiNPL _sdaiGetSubtypes (
#if NeedFunctionPrototypes
    SdaiEntity entity,
    SdaiNPL resultList
#endif
    );

    
/* New */                        
SdaiNPL _sdaiGetExplicitAttrDefinitions (
#if NeedFunctionPrototypes
    SdaiEntity entity,
    SdaiNPL resultList
#endif
    );

/* New */
SdaiNPL _sdaiGetInverseAttrDefinitions (
#if NeedFunctionPrototypes
    SdaiEntity entity,
    SdaiNPL resultList
#endif
    );

/* New */
SdaiNPL _sdaiGetDerivedAttrDefinitions (
#if NeedFunctionPrototypes
    SdaiEntity entity,
    SdaiNPL resultList
#endif
    );

/* New */
SdaiNPL _sdaiGetUniquenessDefinitions (
#if NeedFunctionPrototypes
    SdaiEntity entity,
    SdaiNPL resultList
#endif
    );


SdaiADB *sdaiGetAllAttrs (
#if NeedFunctionPrototypes
    SdaiInstance instance,
    SdaiInteger *numberAttr
#endif	
    );

SdaiAppInstance sdaiNearCopyInstance (
#if NeedFunctionPrototypes
    SdaiAppInstance instance
#endif	
    );

SdaiAppInstance sdaiFarCopyInstance (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiModel mod
#endif	
    );

void sdaiDeleteInstance (
#if NeedFunctionPrototypes
    SdaiAppInstance instance
#endif	
    );

void sdaiPutAttr (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiExplicitAttr attribute,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

void sdaiPutAttrBN (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiString attributeName,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

void sdaiUnsetAttr (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiExplicitAttr attribute
#endif	
    );

void sdaiUnsetAttrBN (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiString attributeName
#endif	
    );

SdaiAggr sdaiCreateAggr (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiExplicitAttr attribute
#endif	
    );

SdaiAggr sdaiCreateAggrBN (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiString attributeName
#endif	
    );

SdaiAggr sdaiCreateAggrADB (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiExplicitAttr attr,
    SdaiADB block
#endif
    );

/* STI Extension function, but really should be in P24 */
SdaiAggr sdaiCreateAggrBNADB (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiString attributeName,
    SdaiADB adb
#endif
    );

/* ISO named this differently than we did, hence the #define here */
#define sdaiCreateAggrADBBN sdaiCreateAggrBNADB

SdaiString sdaiGetPersistentLabel (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiString labelBuffer
#endif	
    );

SdaiAppInstance sdaiGetSessionId (
#if NeedFunctionPrototypes
    SdaiRep repository,
    SdaiString label
#endif	
    );

SdaiString sdaiGetDescription (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiString descriptionBuffer
#endif	
    );

SdaiLogical sdaiValidateWhereRule (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiWhereRule rule
#endif	
    );

/* Validate a rule of a type is a single value */
SdaiLogical _sdaiValidateWhereRule (
#if NeedFunctionPrototypes
    SdaiWhereRule rule,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

SdaiLogical _sdaiValidateUniquenessForInstance (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiUniRule uniRule,
    SdaiSchemaInstance schemaInstance,
    SdaiNPL list
#endif	
    );

    
SdaiLogical sdaiValidateWhereRuleBN (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiString ruleName
#endif	
    );

SdaiBoolean sdaiValidateRequiredAttrs (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiNPL list
#endif	
    );

SdaiBoolean sdaiValidateInverseAttrs (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateAttrTypes (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateAggrSizes (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiNPL list
#endif	
    );

SdaiBoolean sdaiValidateAggrUni (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiNPL list
#endif	
    );

SdaiBoolean sdaiValidateArrayNotOptional (
#if NeedFunctionPrototypes
    SdaiAppInstance instance,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateStringWidth (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateBinaryWidth (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance,
    SdaiNPL list
#endif	
    );

SdaiLogical sdaiValidateRealPrecision (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance,
    SdaiNPL list
#endif	
    );

void sdaiPutAttrs (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance,
    SdaiInteger numberAttr,
    SdaiExplicitAttr attribute,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

void sdaiPutAttrsBN (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance,
    SdaiInteger numberAttr,
    SdaiString attributeName,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

void sdaiPutAllAttrs (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance,
    SdaiInteger numberAttr,
    SdaiADB *values
#endif	
    );

SdaiInteger sdaiGetMemberCount (
#if NeedFunctionPrototypes
    SdaiAggr aggregate
#endif	
    );

SdaiBoolean sdaiIsMember (
#if NeedFunctionPrototypes
    SdaiAggr aggregate,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

SdaiIterator sdaiCreateIterator (
#if NeedFunctionPrototypes
    SdaiAggr aggregate
#endif	
    );

void sdaiDeleteIterator (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

void sdaiBeginning (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

SdaiBoolean sdaiNext (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

void* sdaiGetAggrByIterator (
#if NeedFunctionPrototypes
    SdaiIterator iterator,
    SdaiPrimitiveType valueType,
    void *value
#endif	
    );

SdaiInteger sdaiGetAggrElementBoundByItr (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

SdaiInteger sdaiGetLowerBound (
#if NeedFunctionPrototypes
    SdaiAggr aggregate
#endif	
    );

SdaiInteger sdaiGetUpperBound (
#if NeedFunctionPrototypes
    SdaiAggr aggregate
#endif	
    );

SdaiAggr sdaiCreateNestedAggrByItr (
#if NeedFunctionPrototypes
    SdaiIterator current
#endif	
    );

SdaiAggr sdaiCreateNestedAggrByItrADB (
#if NeedFunctionPrototypes
    SdaiIterator current,
    SdaiADB selaggrInstance
#endif	
    );


void sdaiPutAggrByIterator (
#if NeedFunctionPrototypes
    SdaiIterator iterator,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

void sdaiRemoveByIterator (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

void sdaiAdd (
#if NeedFunctionPrototypes
    SdaiUnorderedAggr unorderedAggr,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

SdaiAggr sdaiCreateNestedAggr (
#if NeedFunctionPrototypes
    SdaiUnorderedAggr aggregate
#endif	
    );

SdaiAggr sdaiCreateNestedAggrADB (
#if NeedFunctionPrototypes
    SdaiUnorderedAggr aggregate,
    SdaiADB selaggrInstance
#endif	
    );

void sdaiRemove (
#if NeedFunctionPrototypes
    SdaiUnorderedAggr unorderedAggr,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

void* sdaiGetAggrByIndex (
#if NeedFunctionPrototypes
    SdaiOrderedAggr aggregate,
    SdaiAggrIndex index,
    SdaiPrimitiveType valueType,
    void *value
#endif	
    );

void sdaiEnd (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

SdaiBoolean sdaiPrevious (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

SdaiInteger sdaiGetAggrElementBoundByIndex (
#if NeedFunctionPrototypes
    SdaiOrderedAggr aggregate,
    SdaiAggrIndex index
#endif	
    );

void sdaiPutAggrByIndex (
#if NeedFunctionPrototypes
    SdaiOrderedAggr aggregate,
    SdaiAggrIndex index,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

SdaiAggr sdaiCreateNestedAggrByIndex (
#if NeedFunctionPrototypes
    SdaiOrderedAggr aggregate,
    SdaiAggrIndex index
#endif	
    );

SdaiAggr sdaiCreateNestedAggrByIndexADB (
#if NeedFunctionPrototypes
    SdaiOrderedAggr aggregate,
    SdaiAggrIndex index,
    SdaiADB selaggrInstance
#endif	
    );

SdaiBoolean sdaiTestArrayByIndex (
#if NeedFunctionPrototypes
    SdaiArray array,
    SdaiAggrIndex index
#endif	
    );

SdaiBoolean sdaiTestArrayByItr (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

SdaiInteger sdaiGetLowerIndex (
#if NeedFunctionPrototypes
    SdaiArray array
#endif	
    );

SdaiInteger sdaiGetUpperIndex (
#if NeedFunctionPrototypes
    SdaiArray array
#endif	
    );

void sdaiUnsetArrayByIndex (
#if NeedFunctionPrototypes
    SdaiArray array,
    SdaiAggrIndex index
#endif	
    );

void sdaiUnsetArrayByItr (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

void sdaiReindexArray (
#if NeedFunctionPrototypes
    SdaiArray array
#endif	
    );

void sdaiResetArrayIndex (
#if NeedFunctionPrototypes
    SdaiArray array,
    SdaiAggrIndex lower,
    SdaiAggrIndex upper
#endif	
    );

void sdaiInsertBefore(
#if NeedFunctionPrototypes
    SdaiIterator iterator,
    SdaiPrimitiveType valueType, ...
#endif	
    );

void sdaiInsertAfter (
#if NeedFunctionPrototypes
    SdaiIterator iterator,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

void sdaiInsertByIndex (
#if NeedFunctionPrototypes
    SdaiList list,
    SdaiAggrIndex index,
    SdaiPrimitiveType valueType,
    ...
#endif	
    );

SdaiAggr sdaiInsertNestedAggrBefore (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

SdaiAggr sdaiInsertNestedAggrBeforeADB (
#if NeedFunctionPrototypes
    SdaiIterator iterator,
    SdaiADB selaggrInstance
#endif	
    );

SdaiAggr sdaiInsertNestedAggrAfter (
#if NeedFunctionPrototypes
    SdaiIterator iterator
#endif	
    );

SdaiAggr sdaiInsertNestedAggrAfterADB (
#if NeedFunctionPrototypes
    SdaiIterator iterator,
    SdaiADB selaggrInstance
#endif	
    );

SdaiAggr sdaiInsertNestedAggrByIndex (
#if NeedFunctionPrototypes
    SdaiList list,
    SdaiAggrIndex index
#endif	
    );

SdaiAggr sdaiInsertNestedAggrByIndexADB (
#if NeedFunctionPrototypes
    SdaiList list,
    SdaiAggrIndex index,
    SdaiADB selaggrInstance
#endif	
    );

void sdaiRemoveByIndex (
#if NeedFunctionPrototypes
    SdaiList list,
    SdaiAggrIndex index
#endif	
    );

void sdaiPutADBTypePath (
#if NeedFunctionPrototypes
    SdaiADB block,
    SdaiInteger typeNameNumber,
    SdaiString * typeNameVector
#endif	
    );

SdaiString * sdaiGetADBTypePath (
#if NeedFunctionPrototypes
    SdaiADB block,
    SdaiInteger * typeNameNumber
#endif	
    );

SdaiLogical sdaiValidateTypePath (
#if NeedFunctionPrototypes
    SdaiAppInstance appInstance,
    SdaiNPL list
#endif	
    );


/********************************************************/
/* STI Extensions */

void _sdaiVerbose (
#if NeedFunctionPrototypes
    SdaiBoolean  print_messages 
#endif	
    );

void _sdaiDisplay (
#if NeedFunctionPrototypes
    SdaiInstance obj
#endif	
    ); 

void _sdaiDisplayAggr (
#if NeedFunctionPrototypes
    SdaiAggr agg
#endif	
    ); 

void _sdaiDisplayADB (
#if NeedFunctionPrototypes
    SdaiADB adb
#endif	
    );

void _sdaiDisplayModel (
#if NeedFunctionPrototypes
    SdaiModel mod
#endif	
    );

SdaiString _sdaiGetErrorName (
#if NeedFunctionPrototypes
    SdaiErrorCode code
#endif	
    );

/* NEW ST-Dev 8.0 */
SdaiInstance _sdaiHeaderDescription (
#if NeedFunctionPrototypes
    SdaiModel model
#endif	
    );

/* NEW ST-Dev 8.0 */
SdaiInstance _sdaiHeaderName (
#if NeedFunctionPrototypes
    SdaiModel model
#endif	
    );
    
    
void _sdaiPreserveIds(
#if NeedFunctionPrototypes
    SdaiModel model,
    SdaiBoolean preserve
#endif	
    );

SdaiBoolean _sdaiGetPreserveIds(
#if NeedFunctionPrototypes
    SdaiModel model
#endif
    );
    
    
void _sdaiGenerateFullDictionary (SdaiBoolean val);
SdaiBoolean _sdaiGetGenerateFullDictionary ();


/* NEW */
int _sdaiGetEntityId (SdaiInstance inst);


typedef unsigned _SdaiPropType;
_SdaiPropType _sdaiGetPropType ();
void _sdaiSetProp (_SdaiPropType type, SdaiAppInstance inst, void * value);
void* _sdaiGetProp (_SdaiPropType type, SdaiAppInstance inst);
void _sdaiRemoveProp (_SdaiPropType type, SdaiAppInstance inst);

    
#ifdef __cplusplus    
}


#ifdef ROSE_H
/* ROSE interface functions.  (Be sure include rose.h to get them) */

RoseStructure* _sdaiGetRoseStructure (SdaiAppInstance inst);

RoseAggregate* _sdaiGetRoseAggregate (SdaiAggr agg);

SdaiAppInstance _sdaiGetAppInstance (RoseStructure * obj);

SdaiAggr _sdaiGetAggr (RoseAggregate * obj);

/* patch 003 */
RoseDesign * _sdaiGetRoseDesign (SdaiModel mod);

/* patch 003 */
SdaiModel _sdaiGetModel (RoseDesign * des);


/* patch 007 convert between ROSE and SDAI meta-data*/
SdaiEntity _sdaiGetEntityForDomain (RoseDomain * dom);
RoseDomain * _sdaiGetDomainForEntity (SdaiEntity ent);

#endif /* ROSE_H */
#endif /* C++ */

#endif

